"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const puppeteer = require("puppeteer");
const _ = require("lodash");
const grpc_plugin_1 = require("./plugin/v1/grpc_plugin");
const grpc_plugin_2 = require("./plugin/v2/grpc_plugin");
const http_server_1 = require("./service/http-server");
const logger_1 = require("./logger");
const browser_1 = require("./browser");
const minimist = require("minimist");
const config_1 = require("./config");
const node_plugin_1 = require("./node-plugin");
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const argv = minimist(process.argv.slice(2));
        const env = Object.assign({}, process.env);
        const command = argv._[0];
        if (command === undefined) {
            const logger = new logger_1.PluginLogger();
            const config = config_1.defaultPluginConfig;
            populatePluginConfigFromEnv(config, env);
            if (!config.rendering.chromeBin && process.pkg) {
                const parts = puppeteer.executablePath().split(path.sep);
                while (!parts[0].startsWith('chrome-')) {
                    parts.shift();
                }
                config.rendering.chromeBin = [path.dirname(process.execPath), ...parts].join(path.sep);
            }
            yield node_plugin_1.serve({
                handshakeConfig: {
                    protocolVersion: 2,
                    magicCookieKey: 'grafana_plugin_type',
                    magicCookieValue: 'datasource',
                },
                versionedPlugins: {
                    1: {
                        'grafana-image-renderer': new grpc_plugin_1.RenderGRPCPluginV1(config, logger),
                    },
                    2: {
                        renderer: new grpc_plugin_2.RenderGRPCPluginV2(config, logger),
                    },
                },
                logger: logger,
                grpcHost: config.plugin.grpc.host,
                grpcPort: config.plugin.grpc.port,
            });
        }
        else if (command === 'server') {
            let config = config_1.defaultServiceConfig;
            if (argv.config) {
                try {
                    const fileConfig = config_1.readJSONFileSync(argv.config);
                    config = _.merge(config, fileConfig);
                }
                catch (e) {
                    console.error('failed to read config from path', argv.config, 'error', e);
                    return;
                }
            }
            populateServiceConfigFromEnv(config, env);
            const logger = new logger_1.ConsoleLogger(config.service.logging);
            const browser = browser_1.createBrowser(config.rendering, logger);
            const server = new http_server_1.HttpServer(config, logger, browser);
            yield server.start();
        }
        else {
            console.log('Unknown command');
        }
    });
}
main().catch(err => {
    console.error(err);
    process.exit(1);
});
function populatePluginConfigFromEnv(config, env) {
    // Plugin v1 legacy env variables
    if (env['GF_RENDERER_PLUGIN_TZ']) {
        config.rendering.timezone = env['GF_RENDERER_PLUGIN_TZ'];
    }
    else {
        config.rendering.timezone = env['TZ'];
    }
    if (env['GF_RENDERER_PLUGIN_GRPC_PORT']) {
        config.plugin.grpc.port = parseInt(env['GF_RENDERER_PLUGIN_GRPC_PORT'], 10);
    }
    if (env['GF_RENDERER_PLUGIN_IGNORE_HTTPS_ERRORS']) {
        config.rendering.ignoresHttpsErrors = env['GF_RENDERER_PLUGIN_IGNORE_HTTPS_ERRORS'] === 'true';
    }
    if (env['GF_RENDERER_PLUGIN_CHROME_BIN']) {
        config.rendering.chromeBin = env['GF_RENDERER_PLUGIN_CHROME_BIN'];
    }
    if (env['GF_RENDERER_PLUGIN_VERBOSE_LOGGING']) {
        config.rendering.verboseLogging = env['GF_RENDERER_PLUGIN_VERBOSE_LOGGING'] === 'true';
    }
    // Plugin v2 env variables needs to be initiated early
    if (env['GF_PLUGIN_GRPC_HOST']) {
        config.plugin.grpc.host = env['GF_PLUGIN_GRPC_HOST'];
    }
    if (env['GF_PLUGIN_GRPC_PORT']) {
        config.plugin.grpc.port = parseInt(env['GF_PLUGIN_GRPC_PORT'], 10);
    }
}
function populateServiceConfigFromEnv(config, env) {
    if (env['BROWSER_TZ']) {
        config.rendering.timezone = env['BROWSER_TZ'];
    }
    else {
        config.rendering.timezone = env['TZ'];
    }
    if (env['HTTP_HOST']) {
        config.service.host = env['HTTP_HOST'];
    }
    if (env['HTTP_PORT']) {
        config.service.port = parseInt(env['HTTP_PORT'], 10);
    }
    if (env['LOG_LEVEL']) {
        config.service.logging.level = env['LOG_LEVEL'];
    }
    if (env['IGNORE_HTTPS_ERRORS']) {
        config.rendering.ignoresHttpsErrors = env['IGNORE_HTTPS_ERRORS'] === 'true';
    }
    if (env['CHROME_BIN']) {
        config.rendering.chromeBin = env['CHROME_BIN'];
    }
    if (env['ENABLE_METRICS']) {
        config.service.metrics.enabled = env['ENABLE_METRICS'] === 'true';
    }
    if (env['RENDERING_MODE']) {
        config.rendering.mode = env['RENDERING_MODE'];
    }
    if (env['RENDERING_CLUSTERING_MODE']) {
        config.rendering.clustering.mode = env['RENDERING_CLUSTERING_MODE'];
    }
    if (env['RENDERING_CLUSTERING_MAX_CONCURRENCY']) {
        config.rendering.clustering.maxConcurrency = parseInt(env['RENDERING_CLUSTERING_MAX_CONCURRENCY'], 10);
    }
    if (env['RENDERING_VERBOSE_LOGGING']) {
        config.rendering.verboseLogging = env['RENDERING_VERBOSE_LOGGING'] === 'true';
    }
    if (env['RENDERING_DUMPIO']) {
        config.rendering.dumpio = env['RENDERING_DUMPIO'] === 'true';
    }
    if (env['RENDERING_ARGS']) {
        const args = env['RENDERING_ARGS'];
        if (args.length > 0) {
            const argsList = args.split(',');
            if (argsList.length > 0) {
                config.rendering.args = argsList;
            }
        }
    }
}
//# sourceMappingURL=app.js.map