"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const grpc = require("@grpc/grpc-js");
const protoLoader = require("@grpc/proto-loader");
const promClient = require("prom-client");
const browser_1 = require("../../browser");
const types_1 = require("./types");
const rendererV2PackageDef = protoLoader.loadSync(__dirname + '/../../../proto/rendererv2.proto', {
    keepCase: true,
    longs: String,
    enums: String,
    defaults: true,
    oneofs: true,
});
const pluginV2PackageDef = protoLoader.loadSync(__dirname + '/../../../proto/pluginv2.proto', {
    keepCase: true,
    longs: String,
    enums: String,
    defaults: true,
    oneofs: true,
});
const rendererV2ProtoDescriptor = grpc.loadPackageDefinition(rendererV2PackageDef);
const pluginV2ProtoDescriptor = grpc.loadPackageDefinition(pluginV2PackageDef);
class RenderGRPCPluginV2 {
    constructor(config, log) {
        this.config = config;
        this.log = log;
        populateConfigFromEnv(this.config);
    }
    grpcServer(server) {
        return __awaiter(this, void 0, void 0, function* () {
            const browser = browser_1.createBrowser(this.config.rendering, this.log);
            const pluginService = new PluginGRPCServer(browser, this.log);
            const rendererServiceDef = rendererV2ProtoDescriptor['pluginextensionv2']['Renderer']['service'];
            server.addService(rendererServiceDef, pluginService);
            const pluginServiceDef = pluginV2ProtoDescriptor['pluginv2']['Diagnostics']['service'];
            server.addService(pluginServiceDef, pluginService);
            const metrics = setupMetrics();
            metrics.up.set(1);
            let browserVersion = 'unknown';
            let labelValue = 1;
            try {
                browserVersion = yield browser.getBrowserVersion();
            }
            catch (err) {
                this.log.error('Failed to get browser version', 'err', err);
                labelValue = 0;
            }
            metrics.browserInfo.labels(browserVersion).set(labelValue);
            if (browserVersion !== 'unknown') {
                this.log.debug('Using browser version', 'browserVersion', browserVersion);
            }
            yield pluginService.start(browserVersion);
        });
    }
}
exports.RenderGRPCPluginV2 = RenderGRPCPluginV2;
class PluginGRPCServer {
    constructor(browser, log) {
        this.browser = browser;
        this.log = log;
    }
    start(browserVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            this.browserVersion = browserVersion;
            yield this.browser.start();
        });
    }
    render(call, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const req = call.request;
            const headers = {};
            if (!req) {
                throw new Error('Request cannot be null');
            }
            if (req.headers) {
                for (const key in req.headers) {
                    if (req.headers.hasOwnProperty(key)) {
                        const h = req.headers[key];
                        headers[key] = h.values.join(';');
                    }
                }
            }
            const options = {
                url: req.url,
                width: req.width,
                height: req.height,
                filePath: req.filePath,
                timeout: req.timeout,
                renderKey: req.renderKey,
                domain: req.domain,
                timezone: req.timezone,
                deviceScaleFactor: req.deviceScaleFactor,
                headers: headers,
            };
            this.log.debug('Render request received', 'url', options.url);
            let errStr = '';
            try {
                yield this.browser.render(options);
            }
            catch (err) {
                this.log.error('Render request failed', 'url', options.url, 'error', err.toString());
                errStr = err.toString();
            }
            callback(null, { error: errStr });
        });
    }
    checkHealth(_, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const jsonDetails = Buffer.from(JSON.stringify({
                browserVersion: this.browserVersion,
            }));
            callback(null, { status: types_1.HealthStatus.OK, message: 'Success', jsonDetails: jsonDetails });
        });
    }
    collectMetrics(_, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = Buffer.from(promClient.register.metrics());
            callback(null, { metrics: { prometheus: payload } });
        });
    }
}
const populateConfigFromEnv = (config) => {
    const env = Object.assign({}, process.env);
    if (env['GF_PLUGIN_RENDERING_CHROME_BIN']) {
        config.rendering.chromeBin = env['GF_PLUGIN_RENDERING_CHROME_BIN'];
    }
    if (env['GF_PLUGIN_RENDERING_ARGS']) {
        const args = env['GF_PLUGIN_RENDERING_ARGS'];
        if (args.length > 0) {
            const argsList = args.split(',');
            if (argsList.length > 0) {
                config.rendering.args = argsList;
            }
        }
    }
    if (env['GF_PLUGIN_RENDERING_IGNORE_HTTPS_ERRORS']) {
        config.rendering.ignoresHttpsErrors = env['GF_PLUGIN_RENDERING_IGNORE_HTTPS_ERRORS'] === 'true';
    }
    if (env['GF_PLUGIN_RENDERING_TIMEZONE']) {
        config.rendering.timezone = env['GF_PLUGIN_RENDERING_TIMEZONE'];
    }
    if (env['GF_PLUGIN_RENDERING_LANGUAGE']) {
        config.rendering.acceptLanguage = env['GF_PLUGIN_RENDERING_LANGUAGE'];
    }
    if (env['GF_PLUGIN_RENDERING_VIEWPORT_WIDTH']) {
        config.rendering.width = parseInt(env['GF_PLUGIN_RENDERING_VIEWPORT_WIDTH'], 10);
    }
    if (env['GF_PLUGIN_RENDERING_VIEWPORT_HEIGHT']) {
        config.rendering.height = parseInt(env['GF_PLUGIN_RENDERING_VIEWPORT_HEIGHT'], 10);
    }
    if (env['GF_PLUGIN_RENDERING_VIEWPORT_DEVICE_SCALE_FACTOR']) {
        config.rendering.deviceScaleFactor = parseFloat(env['GF_PLUGIN_RENDERING_VIEWPORT_DEVICE_SCALE_FACTOR']);
    }
    if (env['GF_PLUGIN_RENDERING_VIEWPORT_MAX_WIDTH']) {
        config.rendering.maxWidth = parseInt(env['GF_PLUGIN_RENDERING_VIEWPORT_MAX_WIDTH'], 10);
    }
    if (env['GF_PLUGIN_RENDERING_VIEWPORT_MAX_HEIGHT']) {
        config.rendering.maxHeight = parseInt(env['GF_PLUGIN_RENDERING_VIEWPORT_MAX_HEIGHT'], 10);
    }
    if (env['GF_PLUGIN_RENDERING_VIEWPORT_MAX_DEVICE_SCALE_FACTOR']) {
        config.rendering.maxDeviceScaleFactor = parseFloat(env['GF_PLUGIN_RENDERING_VIEWPORT_MAX_DEVICE_SCALE_FACTOR']);
    }
    if (env['GF_PLUGIN_RENDERING_MODE']) {
        config.rendering.mode = env['GF_PLUGIN_RENDERING_MODE'];
    }
    if (env['GF_PLUGIN_RENDERING_CLUSTERING_MODE']) {
        config.rendering.clustering.mode = env['GF_PLUGIN_RENDERING_CLUSTERING_MODE'];
    }
    if (env['GF_PLUGIN_RENDERING_CLUSTERING_MAX_CONCURRENCY']) {
        config.rendering.clustering.maxConcurrency = parseInt(env['GF_PLUGIN_RENDERING_CLUSTERING_MAX_CONCURRENCY'], 10);
    }
    if (env['GF_PLUGIN_RENDERING_VERBOSE_LOGGING']) {
        config.rendering.verboseLogging = env['GF_PLUGIN_RENDERING_VERBOSE_LOGGING'] === 'true';
    }
    if (env['GF_PLUGIN_RENDERING_DUMPIO']) {
        config.rendering.dumpio = env['GF_PLUGIN_RENDERING_DUMPIO'] === 'true';
    }
};
const setupMetrics = () => {
    promClient.collectDefaultMetrics();
    const up = new promClient.Gauge({
        name: 'up',
        help: '1 = up, 0 = not up',
    });
    const browserInfo = new promClient.Gauge({
        name: 'grafana_image_renderer_browser_info',
        help: "A metric with a constant '1 value labeled by version of the browser in use",
        labelNames: ['version'],
    });
    return {
        up,
        browserInfo,
    };
};
//# sourceMappingURL=grpc_plugin.js.map